@@ -526,6 +526,7 @@ mod tests {
     use super::*;
     use colors::{hex_to_color_arr, hex_to_color_wgpu};
     use std::io::Write;
+    use sugarloaf::font::fonts::parse_unicode;
 
     fn tmp_dir() -> PathBuf {
         std::env::temp_dir()
@@ -1041,4 +1042,35 @@ mod tests {
         assert_eq!(result.colors.tabs_active, colors::defaults::tabs_active());
         assert_eq!(result.colors.cursor, colors::defaults::cursor());
     }
+
+    #[test]
+    fn test_symbol_map() {
+        let result = create_temporary_config(
+            "symbol-map",
+            r#"
+            fonts.symbol-map = [
+                # covers: '⊗','⊘','⊙'
+                { start = "2297", end = "2299", font-family = "PowerlineSymbols" },
+                { start = "E0C0", end = "E0C7", font-family = "Cascadia Code NF" },
+            ]
+        "#,
+        );
+
+        assert!(result.fonts.symbol_map.is_some());
+        let symbol_map = result.fonts.symbol_map.unwrap();
+        assert_eq!(symbol_map.len(), 2);
+        assert_eq!(symbol_map[0].font_family, "PowerlineSymbols");
+        assert_eq!(symbol_map[0].start, "2297");
+        assert_eq!(symbol_map[0].end, "2299");
+
+        assert_eq!(parse_unicode(&symbol_map[0].start), Some('\u{2297}'));
+        assert_eq!(parse_unicode(&symbol_map[0].end), Some('\u{2299}'));
+
+        assert_eq!(symbol_map[1].font_family, "Cascadia Code NF");
+        assert_eq!(symbol_map[1].start, "E0C0");
+        assert_eq!(symbol_map[1].end, "E0C7");
+
+        assert_eq!(parse_unicode(&symbol_map[1].start), Some('\u{E0C0}'));
+        assert_eq!(parse_unicode(&symbol_map[1].end), Some('\u{E0C7}'));
+    }
 }
