@@ -5,6 +5,9 @@ pub mod font_introspector;
 pub mod layout;
 mod sugarloaf;
 
+// Expose WGPU
+pub use wgpu;
+
 pub use font_introspector::{Stretch, Style, Weight};
 
 pub use crate::sugarloaf::{
@@ -20,5 +23,6 @@ pub use crate::sugarloaf::{
 pub use components::quad::{ComposedQuad, Quad};
 pub use components::rect::Rect;
 pub use layout::{
-    Content, FragmentStyle, FragmentStyleDecoration, UnderlineInfo, UnderlineShape,
+    Content, FragmentStyle, FragmentStyleDecoration, SugarDimensions, UnderlineInfo,
+    UnderlineShape,
 };
